local sin,cos = math.sin,math.cos
require("spriteanim")
require("boxcaster")
require("raycaster")
require("sprite")

Entity = {}
Entity.__index = Entity
Entity.qlist = Game.assets.charquads.ghost
Entity.mul_fwd = 5
Entity.mul_strf = 5
Entity.maxhealth = 5
Entity.ident = "base_entity"
function Entity:new()
	local self = setmetatable({},Entity)
	self.sprite = StaticSprite:new(game.level.sb.entities,self.qlist[1])
	self.anim = SpriteAnim:new(self.sprite,self.qlist)
	self.m_fwd = 0
	self.m_strf= 0
	self.dir = 0
	self.health = self.maxhealth
	self.sprite:setEntityOffset(-16,-32)
	self:createBaseBody()
	return self
end

function Entity:setPosition(x,y)
	self.body:setPosition(x,y)
end

function Entity:getPosition()
	return self.body:getPosition()
end

function Entity:hit() end

function Entity:createBaseBody()
	self.body = love.physics.newBody(game.level.physics,0,0,"dynamic")
	self.body:setFixedRotation(true)
	self.body:setLinearDamping(3)
	self.shape = love.physics.newRectangleShape(24,80)
	self.fixture = love.physics.newFixture(self.body,self.shape,1)
	self.fixture:setUserData(self)
end

function Entity:keypressed(key,ir)
	if key == "w" then
		self.m_fwd = 1*math.max(game.player.level,1)
	elseif key == "a" then
		self.m_strf = -1*math.max(game.player.level,1)
	elseif key == "s" then
		self.m_fwd = -1*math.max(game.player.level,1)
	elseif key == "d" then
		self.m_strf = 1*math.max(game.player.level,1)
	end
end
function Entity:keyreleased(key)
	if key == "w" or key == "s" then
		self.m_fwd = 0
	end
	if key == "d" or key == "a"  then
		self.m_strf = 0
	end
end
function Entity:mousepressed(x,y,b) print(b) end
function Entity:mousereleased(b,x,y) end
function Entity:setAngle(a)	self.body:setAngle(a) end
function Entity:update(dt)
	local px,py = self:getPosition()
	local spc = #self.sprites
	if spc>0 then
		for i=1,spc do
			self.sprites[i]:setPosition(px,py)
		end
	end

	if self.dir >= 1.57 and self.dir <= 4.71 then
		self.sprites[1]:setScale(1,1)
	else
		self.sprites[1]:setScale(-1,1)
	end
	
	local fx,fy = sin(self.dir+1.57)*self.m_fwd*self.mul_fwd + sin(self.dir)*self.m_strf*self.mul_strf, cos(self.dir+1.57)*self.m_fwd*self.mul_fwd + cos(self.dir)*self.m_strf*self.mul_strf
	self.body:applyLinearImpulse(fx,fy)
	if self.m_fwd ~= 0 or self.m_strf ~= 0 then 
		self.anim:update(dt)
	end
end

function Entity:getHealth()
	return self.health
end

require("ghost")
require("zombie")

