local l_gfx = love.graphics
local gw,gh = l_gfx:getDimensions()
local uscalw, uscalh = gw/640,gh/480
require("assets")





Game = {}
Game.__index = Game
Game.states = {Menu = 1, Playing = 2, Over = 3}
Game.assets = Assets:new()


function Game:new()
	local self = setmetatable({},Game)
	self.level = Level:new()
	print(tostring(self.level))
	self.camera = Camera:new()
	self.player = {
		controls = nil,
		ghost = nil,
		score = 0,
		multiplier = 1,
		mult_timer = 0,
		transfers = 0,
		level = 0,
		transfer_add = 0
	}
	self.wave = {
		target = 0,
		spawn_tick = 0,
		spawned = 0,
		number = 0
	}
	
	return self
end

function Game:init()
	self.state = self.states.Playing
	self.level:init()
	local g = self.level:spawn(Ghost)
	g:setPosition(400,300)
	self.player.ghost = g
	game.player.controls = g	
	self.player.transfers = 3
	self.player.score = 0
	self.player.streak = 0
	
	self.wave.target = 10
	self.wave.spawned = 0
	self.wave.number = 1
end

function Game:update(dt)
	if self.state == self.states.Playing then
		local wave = self.wave
		self.level:update(dt)
		local px,py = self.player.controls:getPosition()
		self.camera:setPosition(px-gw/2,py-gh/2)
		if wave.spawned<wave.target then
			wave.spawn_tick = wave.spawn_tick - dt
			if wave.spawn_tick <= 0 then
				wave.spawn_tick = 1
				local z = self.level:spawn(Zombie)
				local r = math.max(gw/2,gh/2)
				local a = math.random(0,628)/100
				z:setPosition(px+math.sin(a)*r,py+math.cos(a)*r)
				z.maxhealth = 5 + wave.number
				z.health = z.maxhealth
				wave.spawned = wave.spawned + 1
			end
		else
			if self.player.score >= wave.target - 1  then
				wave.target = math.floor(wave.target*1.5)
				wave.spawned = 0
				wave.number = wave.number + 1
			
			end
		end
		
		if self.player.controls.ident == self.player.ghost.ident then
			self.player.ghost.health = self.player.ghost.health - dt
		else
			self.player.ghost.health = self.player.ghost.maxhealth
		end
		if (self.player.transfers <= 0 and self.player.controls.id == self.player.ghost.id) or self.player.ghost.health<=0 then
			self.state = self.states.Over
		end
	end
end

function Game:draw()
	if self.state == self.states.Playing then
		self.camera:set()
		self.level:draw()
		local px,py = self.player.controls:getPosition()
		local di = self.player.controls.dir+1.57
		local cx,cy = math.sin(di)*32+px,math.cos(di)*32+py
		love.graphics.circle("line",cx,cy,8)
		self.camera:unset()
	end
	self:drawui()
end

function Game:keypressed(key,ir)
	if self.state == self.states.Playing then
		local c = self.player.controls
		if c~= nil then c:keypressed(key,ir) end
	end
end
function Game:keyreleased(key)
	if self.state == self.states.Playing then
		local c = self.player.controls
		if c~= nil then c:keyreleased(key) end
	end
end
function Game:mousepressed(x,y,b) 
	if self.state == self.states.Playing then
		local c = self.player.controls
		if c~= nil then c:mousepressed(x,y,b) end
	elseif self.state == self.states.Over then
		if b == 1 then
			self:init()
		end
	end
end
function Game:mousereleased(x,y,b) end
function Game:mousemoved(x,y)
	if self.state == self.states.Playing then
		local dir = -math.atan2(gh/2-y,gw/2-x)+3.14
		self.player.controls.dir = dir
	end
end

function Game:drawui()
	if self.state == self.states.Playing then
		l_gfx.print("Game over")
		local uiatl = self.assets.ui_atlas
		l_gfx.draw(uiatl,self.assets.ui_quads.health,0,0,0,uscalw,uscalh)
		l_gfx.draw(uiatl,self.assets.ui_quads.score,l_gfx.getWidth()-254*uscalw,0,0,uscalw,uscalh)
		l_gfx.setFont(self.assets.font)
		l_gfx.setColor(192,0,0,255)
		l_gfx.print(self.player.transfers,0,12*uscalh,0,uscalw,uscalh)
		l_gfx.print("K:"..self.player.score.." W:"..self.wave.number.." S:"..self.player.streak,l_gfx.getWidth()-224*uscalw,0,0,uscalw,uscalh)
		if self.player.controls.ident == self.player.ghost.ident then
			l_gfx.setColor(0,0,192,255)
		end
		l_gfx.rectangle("fill",3*uscalw,4*uscalh,(self.player.controls:getHealth()/self.player.controls.maxhealth)*241*uscalw,8*uscalh)
		
		l_gfx.setColor(255,255,255,255)
		for i=1,self.player.level do
			l_gfx.draw(uiatl,self.assets.ui_quads.lvlbar,((i-1)*18+96)*uscalw,22*uscalh,0,uscalw,uscalh)
		end
	elseif self.state == self.states.Over then
		l_gfx.print("Game over",gw/2-32,gh/2)
	end
end

require("level")
require("camera")