Zombie = {}
Zombie.__index = Zombie
Zombie.qlist = { Game.assets.charquads.zombie, Game.assets.charquads.zombie_atk }
Zombie.ident = "entity_zombie"
Zombie.scan_t = 4
Zombie.punch_power = 1
Zombie.mul_fwd = 10
Zombie.mul_straf = 10
function Zombie:new()
	local self = setmetatable({},Zombie)
	self.sprites = {
		StaticSprite:new(game.level.sb.entities,self.qlist[1][1],nil,true),
		StaticSprite:new(game.level.sb.entities,game.assets.charquads.hpbar[1],nil,true),
		StaticSprite:new(game.level.sb.entities,game.assets.charquads.hpbar[2],nil,true)
	}
	self.sprites[1]:setOffset(24,48)
	self.sprites[2]:setEntityOffset(-17,-64)
	self.sprites[2]:setScale(34,0.5)
	self.sprites[3]:setEntityOffset(-16,-63)
	self.anim = SpriteAnim:new(self.sprites[1],self.qlist[1])
	self.anim:setRate(4)
	self.health = self.maxhealth
	self.marked = false
	self.m_fwd = 0
	self.m_strf= 0
	self.dir = 0
	self.dead = false
	self.dead_t = 1
	self.atk_t = 0
	self.ghost_leadt = 4
	self.scan = self.scan_t
	self:createBaseBody()
	self.body:setLinearDamping(5)
	self.scanner = BoxCaster:new(game.level.physics)
	self.puncher = RayCaster:new(game.level.physics)
	return self
end
setmetatable(Zombie,{__index = Entity})


function Zombie:hit(dmg)
	self.health = self.health - (dmg or 1)
	if self.health<=0 then 
		game.player.score = game.player.score + 1
		game.player.streak = game.player.streak + 1
		game.player.level = math.min(game.player.level + 1,3)
		game.player.transfer_add = game.player.transfer_add + 1
		if game.player.transfer_add>10 then
			game.player.transfers = game.player.transfers + 1
			game.player.transfer_add = 0
		end
	
		self.dead = true
		self.body:setActive(false)
		
		if game.player.controls.id == self.id then
			self:eject()
		end
		
		
	end
end

function Zombie:mousepressed(x,y,b)
	if b == 1 then
		self:attack(true)
	elseif b == 2 then
		self:hit(self.health)
	end
end

function Zombie:eject()
	self.m_fwd = 0
	self.m_strf = 0
	game.player.level = 0
	game.player.controls = game.player.ghost
	local px,py = self:getPosition()
	game.player.ghost:setPosition(px,py)
	game.player.streak = 0
end

function Zombie:attack(controlled)
	if self.atk_t <= 0 then
		self.atk_t = 0.5
		self.anim:setRate(8)
		self.anim:setQuadset(self.qlist[2])
	end
	local px,py = self:getPosition()
	local tx,ty = px+math.sin(self.dir+1.57)*48,py+math.cos(self.dir+1.57)*self.punch_power*48

	local el = self.puncher:cast(px,py,tx,ty)
	self.puncher:sortAscend()
	if #el>0 then
		local ent = el[math.random(#el)][1]:getUserData()
		ent.body:applyLinearImpulse(math.sin(self.dir+1.57)*self.punch_power*800,math.cos(self.dir+1.57)*self.punch_power*800)
		if controlled == true then
			ent:hit(game.player.level*(1+game.wave.number*0.1))
			self.health = math.min(self.health + math.floor(game.player.streak/10)/10,self.maxhealth)
		end
	end
	
end



function Zombie:update(dt)
	if self.dead == false then
		Entity.update(self,dt)
		if self.ghost_leadt>0 then
			local px,py = self:getPosition()
			local ex,ey = game.player.controls:getPosition()
			self.m_fwd = 0.5
			self.ghost_leadt = self.ghost_leadt - dt
			if self.ghost_leadt<= 0 then
				self.m_fwd = 0
			end
		else
			if self.marked == false then
				if game.player.controls.marked == true and game.player.controls.marked ~= game.player.ghost then
					local px,py = self:getPosition()
					local ex,ey = game.player.controls:getPosition()
					self.m_fwd = 0.75+(game.wave.number*0.2)
				else
					self.m_fwd = 0
				end
				
				self.scan = self.scan-dt
				if self.scan<=0 then
					local px,py = self:getPosition()
				
					self.scan = self.scan_t
					local el = self.scanner:test(px-48,py-48,px+48,py+48)
					for i,v in ipairs(el) do
						if v:getUserData().id == game.player.controls.id then
							v:getUserData():hit(1+game.wave.number*0.2)
						end
					end
				end
			end
			
		

			
			
		end
		
		
		if self.marked == false then
			local ex,ey = game.player.controls:getPosition()
			local px,py = self:getPosition()
			local a = -math.atan2(ey-py,ex-px)
			if a<0 then
				a = (math.pi*2+a)
			else
				a = a
			end
			self.dir = a
		end
		
		if self.atk_t > 0 then
			self.atk_t = math.max(0,self.atk_t - dt)
			if self.atk_t <= 0 then
				self.anim:setQuadset(self.qlist[1])
				self.anim:setRate(4)
			end
		end
	else
		self.dead_t = self.dead_t - dt
		self.sprites[1]:setRotation((1-self.dead_t)*1.57)
		if self.dead_t<=0 then
			for i,v in ipairs(self.sprites) do
				v:setScale(0,0)
			end
			self.body:destroy()
			self.del = true
		end
	end
	
	if (self.health<self.maxhealth and self.dead == false) then
		self.sprites[3]:setScale((self.health/self.maxhealth)*32,0.5)
		self.sprites[2]:setScale(34,0.5)
	else
		self.sprites[3]:setScale(0,0)
		self.sprites[2]:setScale(0,0)
	end
end
